import time
import board
import busio
import digitalio
from adafruit_bme280 import basic as adafruit_bme280
import wifi
import ssl
import socketpool
import adafruit_requests
import os
import math

# Set up I2C communication with the BME280 sensor
i2c = busio.I2C(scl=board.SCL, sda=board.SDA)  # Adjust pins as needed
bme280 = adafruit_bme280.Adafruit_BME280_I2C(i2c, address=0x76)

# Set ThingSpeak settings using environment variable
THINGSPEAK_API_KEY = os.getenv("THINGSPEAK_API_KEY")  # Fetch the API key from environment variable
THINGSPEAK_URL = "https://api.thingspeak.com/update.json"

# Altitude for pressure correction (in meters)
altitude_m = 140

# Constants
L = 0.0065  # Temperature lapse rate in K/m
T0 = 288.15  # Standard temperature at sea level in K
g = 9.80665  # Gravitational acceleration in m/s^2
M = 0.0289644  # Molar mass of Earth's air in kg/mol
R = 8.31447  # Universal gas constant in J/(mol·K)


# Connect to WiFi using environment variables
print("Connecting to WiFi...")
wifi.radio.connect(os.getenv("CIRCUITPY_WIFI_SSID"), os.getenv("CIRCUITPY_WIFI_PASSWORD"))
print("Connected to WiFi!")

# Create a socket pool and initialize the requests module without SSL context
pool = socketpool.SocketPool(wifi.radio)
ssl_context = ssl.create_default_context()  # Creating SSL context for secure connection
requests = adafruit_requests.Session(pool, ssl_context)


def correct_pressure_to_sea_level(pressure, temperature, altitude):
    temperature_k = temperature + 273.15       # Convert temperature to Kelvin
    # Calculate sea level pressure
    sea_level_pressure = pressure * (1 - (L * altitude) / temperature_k) ** (-g * M / (R * L))
    return sea_level_pressure

def send_data_to_thingspeak(temperature, pressure, humidity):
    json_payload = {
        "api_key": THINGSPEAK_API_KEY,
        "field1": temperature,
        "field2": pressure,
        "field3": humidity
    }
    
    try:
        print("Sending data to ThingSpeak...")
        response = requests.post(THINGSPEAK_URL, json=json_payload)
        print("Response: ", response.text)
    except Exception as e:
        print("Failed to send data: ", e)

while True:
    try:
        # Read BME280 sensor data
        temperature = bme280.temperature
        pressure = bme280.pressure
        humidity = bme280.humidity
        
        # Correct pressure to sea level
        corrected_pressure = correct_pressure_to_sea_level(pressure, temperature, altitude_m)
        
        print(f"Temperature: {temperature} C")
        print(f"Pressure (corrected to sea level): {corrected_pressure} hPa")
        print(f"Humidity: {humidity} %")
        
        # Send data to ThingSpeak
        send_data_to_thingspeak(temperature, corrected_pressure, humidity)
        
    except Exception as e:
        print("Error reading sensor data: ", e)
    
    # Wait for 15 seconds before sending the next batch of data
    time.sleep(15)

